//==========================================
// Include file area
//==========================================
.include	GPL951_Body.inc	
.include	CTSDefine.inc

//==========================================
// Public declare area
//==========================================
//======== Function Public =============
.public		F_CTS_User_Init
.public		F_GetOnThreshold
.public		F_GetOffThreshold
.public		F_CTSSetScanNum
.public		F_CTSChSel
.public		F_CTSStart
.public		F_CTSAllChOff

//======== Variable Public =============
.public		R_CtsSum	
.public		R_CtsRefer
.public		R_CtsReferL
.public		R_TriggerLevel
.public		R_ThresholdOn
.public		R_ThresholdOff
.public		R_ScanNumber
.public		R_ScanCycleNum

//======== Table Public =============
.public		T_TotalPadNum
.public		T_TraceUpStep
.public		T_TraceDownStep
.public		T_CalibrationCount
.public		T_ReferenceErrCnt
.public		T_CtsKeyDebounceTime
.public		T_ScanCycleCnt
.public		T_ScanLine
.public		T_OnThresholdLevel
.public		T_OffThresholdLevel

.public		_R_CtsSum
.public		_R_CtsRefer
//==========================================
//Variable RAM declare area
//==========================================
CTS_RAM:	.section	.ram
_R_CtsSum:
R_CtsSum:	
	.dw		D_SinglePadNum		dup(?)
_R_CtsRefer:
R_CtsRefer:	
	.dw		D_SinglePadNum		dup(?)
R_CtsReferL:	
	.dw		D_SinglePadNum		dup(?)
R_TriggerLevel:
	.dw		D_SinglePadNum		dup(?)

R_ThresholdOn:
	.dw		D_SinglePadNum		dup(?)

R_ThresholdOff:
	.dw		D_SinglePadNum		dup(?)

R_ScanNumber:
	.dw		D_SinglePadNum		dup(?)

.var	R_ScanCycleNum

//==========================================
// code starting 
//==========================================
CTS_Code:	.section	.code
// =======================================================================================
// Function name: F_CTS_User_Init
// Purpose		: CTS driver Hardware User initial
// Parameter	: None
// Return		: None
// ======================================================================================
F_CTS_User_Init:
	call	F_CTS_Para_User_Initial
	
	r1 = 0xFFFF
	[P_IOF_Dir] = r1
	[P_IOF_Attrib] = r1
//	[P_IOF_Buffer] = r1

	r1 = [P_LVR_Ctrl]		// Frank add 2016.10.12, IOF[0] will not output scan waveform, when CTS scan done.
	r1 |= 0x1
	[P_LVR_Ctrl] = r1

	r1 = [P_IOF_Mux]		// Frank add 2017.04.27
	r1 |= C_IOF_CTS_En		// Frank modify 2017.04.27
	[P_IOF_Mux] = r1
	r1 = C_CTS_SYS_DIV2
	[P_CTS_DIV] = r1
	r1 = C_CTS_LDO33_En
	[P_CTS_Ctrl3] = r1
	r1 = C_CTS_ScanAllDoneIntEn
	[P_CTS_Ctrl2] = r1
	r1 = C_CTS_Current_600u + C_CTS_Duration_450n + C_CTS_NF_200M + C_CTS_TriEn + D_CtsVrefSel_275_025
	[P_CTS_Ctrl1] = r1
	retf

// =======================================================================================
// Function name: F_CTSPara_User_Initial
// Purpose		: CTS Default Parameters initial
// Parameter	: None
// Return		: None
// =======================================================================================
F_CTS_Para_User_Initial:
	r1 = seg16	T_ScanNum
	ds = r1
	r1 = D_SinglePadNum-1
L_Initial_Para:
	r2 = T_ScanNum
	r2 = r2 + r1
	r3 = DS:[r2]
	r2 = R_ScanNumber
	r2 = r2 + r1
	[r2] = r3
	
	r2 = T_OnThresholdLevel
	r2 = r2 + r1
	r3 = DS:[r2]
	r2 = R_ThresholdOn
	r2 = r2 + r1
	[r2] = r3
	
	r2 = T_OffThresholdLevel
	r2 = r2 + r1
	r3 = DS:[r2]
	r2 = R_ThresholdOff
	r2 = r2 + r1
	[r2] = r3
	
	r1 = r1 - 1
	jpl		L_Initial_Para
	
	r1 = seg16	T_ScanCycleCnt
	ds = r1
	r1 = T_ScanCycleCnt
	r2 = DS:[r1]
	[R_ScanCycleNum] = r2
	retf
	
// =======================================================================================
// Function name: F_CTSSetScanNum
// Purpose		: Set IO Scan Number
// Parameter	: r4: Scan Pad Index
// Return		: None
// =======================================================================================	
F_CTSSetScanNum:
	r2 = R_ScanNumber
	r2 = r2 + r4
	r2 = [r2]
	[P_CTS_CYCLE] = r2
	retf

// =======================================================================================
// Function name: F_GetOnThreshold
// Purpose		: Get On Threshold
// Parameter	: r4 Pad index
// Return		: r1: On Threshold
// =======================================================================================	
F_GetOnThreshold:
	r1 = R_ThresholdOn
	r1 = r1 + r4
	r1 = [r1]
	retf
		
// =======================================================================================
// Function name: F_GetOffThreshold
// Purpose		: Get Off Threshold
// Parameter	: r4: Pad index
// Return		: r1: OFF Threshold
// ======================================================================================		
F_GetOffThreshold:
	r1 = R_ThresholdOff
	r1 = r1 + r4
	r1 = [r1]
	retf
	
// =======================================================================================
// Function name: F_CTSChSel
// Purpose		: Set CTS scan channel to pad index
// Parameter	: r4 is Pad Index
// ======================================================================================
F_CTSChSel:	
	r2 = seg16	T_CTS_IOSel
	ds = r2
	r2 = T_CTS_IOSel
	r2 = r2 + r4
	r2 = ds:[r2]
	[P_CTS_CH] = r2
	retf

// =======================================================================================
// Function name: F_CTSStart
// Purpose		: CTS Start Scan
// Parameter	: None
// Return		: None
// ======================================================================================
F_CTSStart:
	r1 = [P_CTS_Ctrl1]
	r1 = r1 | (C_CTS_En + C_CTS_Start)
	[P_CTS_Ctrl1] = r1
	retf
	
// =======================================================================================
// Function name: F_CTSAllChOff
// Purpose		: Off all CTS channel and driver
// Parameter	: None
// Return		: None
// ======================================================================================
F_CTSAllChOff:
	r1 = [P_CTS_Ctrl1]
	r1 = r1 & ~(C_CTS_En)
	[P_CTS_Ctrl1] = r1
	retf
	
// =======================================================================================
// Common define table
// ======================================================================================
T_TotalPadNum:
	.dw		D_SinglePadNum-1
	.dw		D_SinglePadNum
T_TraceUpStep:
	.dw		D_TraceUpStep
T_TraceDownStep:
	.dw		D_TraceDownStep
T_CtsKeyDebounceTime:
	.dw		D_CtsKeydebounceTime
T_CalibrationCount:
	.dw		D_CalibrationCount
T_ReferenceErrCnt:
	.dw		D_ReferenceErrCnt
T_ScanCycleCnt:
	.dw		D_ScanCycleCnt+2	
// ======================================================================================
T_ScanLine:					//IO pin scan sequence 
	.dw		0x01<<CTS_Pad0
	.dw		0x01<<CTS_Pad1
	.dw		0x01<<CTS_Pad2
	.dw		0x01<<CTS_Pad3
	.dw		0x01<<CTS_Pad4
	.dw		0x01<<CTS_Pad5
	.dw		0x01<<CTS_Pad6
	.dw		0x01<<CTS_Pad7
	.dw		0x01<<CTS_Pad8
	.dw		0x01<<CTS_Pad9
	.dw		0x01<<CTS_Pad10
	.dw		0x01<<CTS_Pad11
	.dw		0x01<<CTS_Pad12
	.dw		0x01<<CTS_Pad13
	.dw		0x01<<CTS_Pad14
	.dw		0x01<<CTS_Pad15
	
// =======================================================================================
// Below tabl
// ======================================================================================		
T_OnThresholdLevel:
	.dw		Pad0_OnLv
	.dw		Pad1_OnLv
	.dw		Pad2_OnLv
	.dw		Pad3_OnLv
	.dw		Pad4_OnLv
	.dw		Pad5_OnLv
	.dw		Pad6_OnLv
	.dw		Pad7_OnLv
	.dw		Pad8_OnLv
	.dw		Pad9_OnLv
	.dw		Pad10_OnLv
	.dw		Pad11_OnLv
	.dw		Pad12_OnLv
	.dw		Pad13_OnLv
	.dw		Pad14_OnLv
	.dw		Pad15_OnLv
	
T_OffThresholdLevel:
	.dw		Pad0_OffLv
	.dw		Pad1_OffLv
	.dw		Pad2_OffLv
	.dw		Pad3_OffLv
	.dw		Pad4_OffLv
	.dw		Pad5_OffLv
	.dw		Pad6_OffLv
	.dw		Pad7_OffLv
	.dw		Pad8_OffLv
	.dw		Pad9_OffLv
	.dw		Pad10_OffLv
	.dw		Pad11_OffLv
	.dw		Pad12_OffLv
	.dw		Pad13_OffLv
	.dw		Pad14_OffLv
	.dw		Pad15_OffLv
	
//============ IO Select Table ======================	
T_CTS_IOSel:
	.dw		CTS_Pad0 + (CTS_Pad0<<4)
	.dw		CTS_Pad1 + (CTS_Pad1<<4)
	.dw		CTS_Pad2 + (CTS_Pad2<<4)
	.dw		CTS_Pad3 + (CTS_Pad3<<4)
	.dw		CTS_Pad4 + (CTS_Pad4<<4)
	.dw		CTS_Pad5 + (CTS_Pad5<<4)
	.dw		CTS_Pad6 + (CTS_Pad6<<4)
	.dw		CTS_Pad7 + (CTS_Pad7<<4)
	.dw		CTS_Pad8 + (CTS_Pad8<<4)
	.dw		CTS_Pad9 + (CTS_Pad9<<4)
	.dw		CTS_Pad10 + (CTS_Pad10<<4)
	.dw		CTS_Pad11 + (CTS_Pad11<<4)
	.dw		CTS_Pad12 + (CTS_Pad12<<4)
	.dw		CTS_Pad13 + (CTS_Pad13<<4)
	.dw		CTS_Pad14 + (CTS_Pad14<<4)
	.dw		CTS_Pad15 + (CTS_Pad15<<4)

//============ Scan Number Table ======================		
T_ScanNum:
	.dw		Pad0_ScanNum
	.dw		Pad1_ScanNum
	.dw		Pad2_ScanNum
	.dw		Pad3_ScanNum
	.dw		Pad4_ScanNum
	.dw		Pad5_ScanNum
	.dw		Pad6_ScanNum
	.dw		Pad7_ScanNum
	.dw		Pad8_ScanNum
	.dw		Pad9_ScanNum
	.dw		Pad10_ScanNum
	.dw		Pad11_ScanNum
	.dw		Pad12_ScanNum
	.dw		Pad13_ScanNum
	.dw		Pad14_ScanNum
	.dw		Pad15_ScanNum